VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSysInfo"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' -------------------------------------------------------------------------------
'
'   AVStoDVD - by MrC
'   File: clsSysInfo.cls
'   Purpose: Class that reads System Info
'
' -------------------------------------------------------------------------------


Option Explicit
Option Compare Binary
Option Base 0


' System Info Type (API GetSystemInfo)
Private Type SYSTEMINFO
   dwOemID As Long
   dwPageSize As Long
   lpMinimumApplicationAddress As Long
   lpMaximumApplicationAddress As Long
   dwActiveProcessorMask As Long
   dwNumberOfProcessors As Long
   dwProcessorType As Long
   dwAllocationGranularity As Long
   wProcessorLevel As Integer
   wProcessorRevision As Integer
End Type

' OS version type (API GetVersionEx)
Private Type OSVERSIONINFOEX
    dwOSVersionInfoSize As Long
    dwMajorVersion As Long
    dwMinorVersion As Long
    dwBuildNumber As Long
    dwPlatformId As Long
    szCSDVersion As String * 128
    wServicePackMajor As Integer
    wServicePackMinor As Integer
    wSuiteMask As Integer
    wProductType As Byte
    wReserved As Byte
End Type
    
' Memory Status Type (API GlobalMemoryStatus)
Private Type MEMORYSTATUS
    dwLength As Long
    dwMemoryLoad As Long
    dwTotalPhys As Long
    dwAvailPhys As Long
    dwTotalPageFile As Long
    dwAvailPageFile As Long
    dwTotalVirtual As Long
    dwAvailVirtual As Long
End Type

' Enum type for API SHGetFolderPath
Public Enum CSIDL_SPECIALPATHCODE
    CSIDL_USER_STARTMENUPROGRAMS = 2            '{user}\Start Menu\Programs
    CSIDL_USER_DOCUMENTS = 5                    '{user}\My Documents
    CSIDL_USER_FAVORITES = 6                    '{user}\My Favorites
    CSIDL_USER_STARTMENUPROGRAMSSTARTUP = 7     '{user}\Start Menu\Programs\Startup
    CSIDL_USER_RECENTS = 8                      '{user}\My Recents
    CSIDL_USER_SENDTO = 9                       '{user}\SendTo
    CSIDL_USER_STARTMENU = 11                   '{user}\Start Menu
    CSIDL_USER_MUSIC = 13                       '{user}\My Music
    CSIDL_USER_VIDEOS = 14                      '{user}\My Videos
    CSIDL_USER_DESKTOP = 16                     '{user}\Desktop
    CSIDL_USER_NETWORK = 19                     '{user}\My Network
    CSIDL_FONTS = 20                            'System fonts
    CSIDL_USER_MODELS = 21                      '{user}\My Models
    CSIDL_COMMON_STARTMENU = 22                 '{all user}\Start Menu
    CSIDL_COMMON_STARTMENUPROGRAMS = 23         '{all user}\Start Menu\Programs
    CSIDL_COMMON_STARTMENUPROGRAMSSTARTUP = 24  '{all user}\Start Menu\Programs\Startup
    CSIDL_COMMON_DESKTOP = 25                   '{all user}\Desktop
    CSIDL_USER_APPDATA = 26                     '{user}\Application Data
    CSIDL_USER_PRINTERS = 27                    '{user}\Printers
    CSIDL_USER_LOCAL_APPDATA = 28               '{user}\Local Settings\Application Data
    CSIDL_COMMON_FAVORITES = 31                 '{all user}\Favorites
    CSIDL_USER_LOCAL_INTERNETCACHE = 32         '{user}\Local Settings\Internet Temporary Files
    CSIDL_USER_COOKIES = 33                     '{user}\My Cookies
    CSIDL_USER_LOCAL_HISTORY = 34               '{user}\Local Settings\History
    CSIDL_COMMON_APPDATA = 35                   '{all user}\Application Data
    CSIDL_WINDOWS = 36                          'Windows folder
    CSIDL_SYSTEM = 37                           'System folder
    CSIDL_PROGRAMFILES = 38                     'Program Files folder
    CSIDL_USER_PICTURES = 39                    '{user}\My Pictures
    CSIDL_USER = 40                             '{user} folder
    CSIDL_PROGRAMFILES_COMMON = 43              'Program Files\Common
    CSIDL_COMMON_MODELS = 45                    '{all user}\My Models
    CSIDL_COMMON_DOCUMENTS = 46                 '{all user}\My Documents
    CSIDL_COMMON_STARTMENUPROGRAMSADMTOOLS = 47 '{all user}\Start Menu\Programs\Administrative Tools
    CSIDL_COMMON_MUSIC = 53                     '{all user}\My Music
    CSIDL_COMMON_PICTURES = 54                  '{all user}\My Pictures
    CSIDL_COMMON_VIDEOS = 55                    '{all user}\My Videos
    CSIDL_RESOURCES = 56                        'Windows Resources folder
End Enum

' Const for API SHGetFolderPath
Private Const CSIDL_FLAG_CREATE = &H8000&               'combine with CSIDL_ value to force
Private Const CSIDL_FLAG_MASK = &HFF00                  'mask for all possible flag values
Private Const SHGFP_TYPE_CURRENT = &H0                  'current value for user, verify it exists
Private Const SHGFP_TYPE_DEFAULT = &H1
Private Const CSIDL_INVALIDARG = &H80070057             'Invalid CSIDL Value

' Windows NT 5 version
Private Const VER_PLATFORM_WIN32_NT = 2

Private SysInfo As SYSTEMINFO
Private myOS As OSVERSIONINFOEX
Private ms As MEMORYSTATUS
   
' API calls
Private Declare Sub GetSystemInfo Lib "kernel32" (lpSystemInfo As SYSTEMINFO)
Private Declare Sub GlobalMemoryStatus Lib "kernel32" (lpBuffer As MEMORYSTATUS)
Private Declare Function SHGetFolderPath Lib "shfolder.dll" Alias "SHGetFolderPathA" (ByVal hwndOwner As Long, ByVal nFolder As Long, ByVal hToken As Long, ByVal dwFlags As Long, ByVal pszPath As String) As Long
Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" (ByRef lpVersionInformation As OSVERSIONINFOEX) As Long
Private Declare Function GetSystemDirectory Lib "kernel32" Alias "GetSystemDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Private Declare Function GetWindowsDirectory Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long


Private Sub Class_Initialize()

    ' Get CPU information
    GetSystemInfo SysInfo
    
    ' Get OS information
    myOS.dwOSVersionInfoSize = Len(myOS)
    GetVersionEx myOS
    
    ' Get ms information
    ms.dwLength = LenB(ms)
    GlobalMemoryStatus ms

End Sub


' Get CPU Name (Needs module modReg.bas and modStr.bas)
Public Property Get ProcessorName() As String
    
    ProcessorName = modStr.RemoveCharBegEnd(modReg.EmiLeggiChiaveStringa("HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0", "ProcessorNameString", ""), " ")
      
End Property


' Get CPU ID (Needs module modReg.bas and modStr.bas)
Public Property Get ProcessorID() As String
    
    ProcessorID = modStr.RemoveCharBegEnd(modReg.EmiLeggiChiaveStringa("HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0", "Identifier", ""), " ")
    If ProcessorID = "" Then
        ProcessorID = "Type " & SysInfo.dwProcessorType & ", Family " & SysInfo.wProcessorLevel & ", Revision " & SysInfo.wProcessorRevision & ", Model " & HiByte(SysInfo.wProcessorRevision) & ", Stepping " & LoByte(SysInfo.wProcessorRevision)
    End If
      
End Property


' Get CPU Speed (Needs module modReg.bas)
Public Property Get ProcessorSpeed() As Long
    
    ProcessorSpeed = modReg.EmiLeggiChiaveNumero("HKEY_LOCAL_MACHINE\HARDWARE\DESCRIPTION\System\CentralProcessor\0", "~MHz", 0)
      
End Property


' Get number of CPU (processors)
Public Property Get ProcessorNumber() As Long
    
    ProcessorNumber = SysInfo.dwNumberOfProcessors
      
End Property


' Determine if the current OS is at least NT version 5 (Windows 2000 and higher)
Public Property Get OSIsNT5() As Boolean

    On Error Resume Next
    
    OSIsNT5 = False
    If myOS.dwPlatformId = VER_PLATFORM_WIN32_NT And myOS.dwMajorVersion >= 5 Then OSIsNT5 = True
    
End Property


' Return current OS version + build
Public Property Get OSVersion() As String

    '               Win95   Win95   Win98   Win98   WinMe   WinNT4  Win2k   WinXP   Win2k3  Win     Win7    Win8
    '                       OSR2            SE                                      Server  Vista
    'PlatformID     1       1       1       1       1       2       2       2       2       2       2       2
    'Major Version  4       4       4       4       4       4       5       5       5       6       6       6
    'Minor Version  0       0       10      10      90      0       0       1       2       0       1       2
    'Build          950*    1111    1998    2222    3000    1381    2195    2600    3790    ****    ****    ****
    
    If myOS.dwPlatformId = 1 Then
        If myOS.dwMajorVersion = 4 Then
            Select Case myOS.dwMinorVersion
            Case 0
                If InStr(1, myOS.dwBuildNumber, 950) <> 0 Then
                    OSVersion = "Windows 95"
                ElseIf myOS.dwBuildNumber = 1111 Then
                    OSVersion = "Windows 95 OSR2"
                End If
            Case 10
                Select Case myOS.dwBuildNumber
                Case 1998
                    OSVersion = "Windows 98"
                Case 2222
                    OSVersion = "Windows 98 SE"
                End Select
            Case 90
                OSVersion = "Windows Me"
            End Select
        End If
    ElseIf myOS.dwPlatformId = 2 Then
        Select Case myOS.dwMajorVersion
        Case 4
            OSVersion = "Windows NT4"
        Case 5
            Select Case myOS.dwMinorVersion
            Case 0
                OSVersion = "Windows 2000"
            Case 1
                OSVersion = "Windows XP"
            Case 2
                OSVersion = "Windows 2003 Server"
            End Select
        Case 6
            Select Case myOS.dwMinorVersion
            Case 0
                OSVersion = "Windows Vista"
            Case 1
                OSVersion = "Windows 7"
            Case 2
                OSVersion = "Windows 8"
            End Select
        End Select
    End If
    If OSVersion = "" Then
        OSVersion = "Windows " & myOS.dwMajorVersion & "." & myOS.dwMinorVersion & " build " & myOS.dwBuildNumber
    End If
    If myOS.szCSDVersion <> "" Then
        OSVersion = OSVersion & " " & StripNull(myOS.szCSDVersion)
    End If
    
End Property


' Return Total Phisical Memory installed
Public Property Get PhisicalMemoryTotal() As Long

   PhisicalMemoryTotal = ms.dwTotalPhys
   
End Property


' Return Available Phisical Memory
Public Property Get PhisicalMemoryAvailable() As Long

   PhisicalMemoryAvailable = ms.dwAvailPhys
   
End Property


' Return Windows folder path
Public Property Get PathWindows() As String
   
   Dim WinDir As String
   Dim ResultCode As Long

   WinDir = Space$(260)
   ResultCode = GetWindowsDirectory(WinDir, 259)
   WinDir = Left$(WinDir, ResultCode)
   PathWindows = Trim$(WinDir)

End Property


' Return Windows SYSTEM folder path
Public Property Get PathSystem() As String
   
    Dim SysDir As String
    Dim ResultCode As Long

    On Error GoTo Err_Handling

    SysDir = Space$(260)
    ResultCode = GetSystemDirectory(SysDir, 259)
    SysDir = Left$(SysDir, ResultCode)
    PathSystem = Trim$(SysDir)
    
    Exit Property
    
Err_Handling:
PathSystem = ""

End Property


' Return Windows PROGRAMS folder path
Public Property Get PathPrograms() As String
   
    Dim ProgDir As String

    On Error GoTo Err_Handling

    ProgDir = SpecialPath(CSIDL_PROGRAMFILES)
    PathPrograms = Trim$(ProgDir)
    
    Exit Property
    
Err_Handling:
PathPrograms = ""

End Property


' Return Windows Special folder path. Return SystemDrive if SHGetFolderPath is unavailable
Public Property Get SpecialPath(SpecialPathCode As CSIDL_SPECIALPATHCODE) As String

    Dim lngReturn As Long
    Dim strPath As String

    On Error GoTo Err_Handling
    
    strPath = String(260, 0)
    lngReturn = SHGetFolderPath(0, SpecialPathCode, 0, SHGFP_TYPE_CURRENT, strPath)
    Select Case lngReturn
        Case 0  'OK
            SpecialPath = Left$(strPath, InStr(1, strPath, Chr(0)) - 1)
        Case 1  'Folder Does Not Exist
        Case CSIDL_INVALIDARG  'Folder Not Valid on this OS
    End Select
    If SpecialPath = "" Then SpecialPath = Environ("SystemDrive")

    Exit Property

Err_Handling:
SpecialPath = Environ("SystemDrive")

End Property


' Shutdown system using Windows built in commands
Public Sub WinShutdown(Time As Long, Str As String)

    On Error GoTo Err_Handling
    
    If InStr(1, OSVersion, "Windows 95") <> 0 Then
        Execute PathSystem & "\rundll.exe", "USER.EXE,ExitWindows", SW_DEFAULT, False
    ElseIf InStr(1, OSVersion, "Windows 98") <> 0 Or InStr(1, OSVersion, "Windows Me") <> 0 Then
        Execute PathSystem & "\rundll32.exe", "SHELL32.DLL,SHExitWindowsEx 1", SW_DEFAULT, False
    ElseIf InStr(1, OSVersion, "Windows NT") <> 0 Or InStr(1, OSVersion, "Windows 2000") <> 0 Then
        Execute PathSystem & "\shutdown.exe", "/L /T:" & Time & " /Y /C", SW_DEFAULT, False
    ElseIf InStr(1, OSVersion, "Windows XP") <> 0 Or InStr(1, OSVersion, "Windows 2003") <> 0 Or InStr(1, OSVersion, "Windows Vista") <> 0 Or InStr(1, OSVersion, "Windows 7") <> 0 Or InStr(1, OSVersion, "Windows 8") <> 0 Then
        Execute PathSystem & "\shutdown.exe", "-s -t " & Time & " -f -c """ & Str & """", SW_DEFAULT, False
    Else
        GoTo Err_Handling
    End If
    
    Exit Sub
    
Err_Handling:
MsgBox "Cannot execute shutdown.", vbInformation, App.Title
    
End Sub


Private Function HiByte(ByVal wParam As Integer) As Byte
   
  'note: VB4-32 users should declare this function As Integer
   HiByte = (wParam And &HFF00&) \ (&H100)

End Function


Private Function LoByte(ByVal wParam As Integer) As Byte

  'note: VB4-32 users should declare this function As Integer
   LoByte = wParam And &HFF&

End Function


