Attribute VB_Name = "modAVS"
' -------------------------------------------------------------------------------
'
'   AVStoDVD - by MrC
'   File: modAVS.bas
'   Purpose: Create AviSynth script
'   Requires: modFS.bas, modNum.bas
'
' -------------------------------------------------------------------------------


Option Explicit
Option Compare Binary
Option Base 0


Public Function Create_AVSScript(Title As TitleSettings, Optional AudioNum As Integer = 1) As String

    Dim X As Single
    Dim Y As Single
    Dim dX As Single
    Dim dY As Single
    Dim FPS As Single
    Dim InputFPS As Single
    Dim InputDAR As Single
    Dim InputInterlaced As Boolean
    Dim DVD_V As Integer
    Dim DVD_H As Integer
    Dim AVSAudioInput As String
    Dim Str As String
    Dim VideoStr As String
    Dim AudioStr As String
    Dim ResizeStr As String
    Dim ResizeFilter As String
    Dim VideoStandardStr As String
    Dim Flag As Boolean
    Dim sPlugin As String
    Dim TrackNum As Integer

    On Error GoTo Err_Handling
    
    With Title
        ' Initialize variables
        DVD_H = 352
        If .VideoResolution = Full_D1 Then DVD_H = 720
        VideoStandardStr = "pal_video"
        FPS = 25
        DVD_V = 576
        If VideoStandard = NTSC Then
            VideoStandardStr = "ntsc_video"
            FPS = 2997 / 100
            DVD_V = 480
        End If
        If .VideoResolution = VCD Then DVD_V = DVD_V / 2
        InputFPS = Val(.InputVideoInfo.FPS)
        If InputFPS = 0 Then InputFPS = FPS
        InputDAR = Val(.InputVideoInfo.DAR)
        If InputDAR = 0 Then InputDAR = 1
        If Val(.InputVideoInfo.Height) / DVD_H * Val(.InputVideoInfo.Width) / DVD_V < 1 Then
            ResizeFilter = Preferences.AVSUpSize
        Else
            ResizeFilter = Preferences.AVSDownSize
        End If
        If InStr(UCase(.InputVideoInfo.ScanType), "INTERLACED") <> 0 Then
            InputInterlaced = True
        End If
        
        ' Set Video Source
        If .VideoIsToBeKept = True Then
            VideoStr = VideoStr & "# Video is frameserved by AviSynth just for Preview and Edit purposes." & vbCrLf
        End If
        Select Case .InputVideoInfo.Compression
        Case "AviSynth"
            VideoStr = VideoStr & "Video = Import(""" & .InputVideoPathName & """)" & vbCrLf
        Case "DGIndex"
            VideoStr = VideoStr & "Video = MPEG2Source(""" & .InputVideoPathName & """)" & vbCrLf
        Case Else
            Select Case Preferences.AVSVideoInput
            Case "AVISource"
                VideoStr = VideoStr & "Video = AVISource(""" & .InputVideoPathName & """, audio=false)" & vbCrLf
            Case "DirectShowSource"
                VideoStr = VideoStr & "Video = DirectShowSource(""" & .InputVideoPathName & """, "
                If InputFPS > 0 Then
                    VideoStr = VideoStr & "fps=" & Replace(CStr(InputFPS), ",", ".") & ", "
                End If
                If .InputVideoInfo.FrameRateMode <> "CFR" Then
                    VideoStr = VideoStr & "convertfps=true, "
                End If
                VideoStr = VideoStr & "audio=false)" & vbCrLf
            Case "FFMpegSource"
                VideoStr = VideoStr & "Video = FFVideoSource(""" & .InputVideoPathName & """, track=-1, seekmode=0)" & vbCrLf
            Case Else
                VideoStr = VideoStr & "Video = A2DVideoSource(""" & .InputVideoPathName & """, CacheFolder=""" & Preferences.TempPath & """, FrameRate=" & Replace(CStr(InputFPS), ",", ".") & ", VFR=" & IIf(.InputVideoInfo.FrameRateMode = "CFR", "false", "true") & ")" & vbCrLf
            End Select
        End Select
        
        ' Set Audio Source
        If .InputAudioPathName(AudioNum) <> "" Then     'And .AudioIsToBeKept = False
            Flag = False
            If .AudioIsToBeKept(AudioNum) = True Then Flag = True
            If .AudioFormat(AudioNum) = AC3 And Preferences.AC3AudioEncoder = FFmpeg_Safe Then Flag = True
            If .TimeTrimmed > 7200 And .AudioChannelsMode(AudioNum) = "5.1" And Preferences.ForceFFmpeg4GB Then Flag = True
            If Flag = True Then
                AudioStr = AudioStr & "# Audio is frameserved by AviSynth just for Preview and Edit purposes." & vbCrLf
            End If
            If .InputAudioInfo(AudioNum).Compression <> "AviSynth" Then
                AVSAudioInput = Preferences.AVSAudioInput
                TrackNum = Val(.InputAudioInfo(AudioNum).TrackID)
                If TrackNum > 1 Then
                    AVSAudioInput = "FFAudioSource (forced)"
                    AudioStr = AudioStr & "# Forced 'FFAudioSource' to encode secondary Audio Tracks" & vbCrLf
                End If
                If .InputAudioInfo(AudioNum).Compression = "AC3" And AVSAudioInput = "AVISource" Then
                    AVSAudioInput = "DirectShowSource"
                    AudioStr = AudioStr & "# 'AVISource' replaced with 'DirectShowSource' for AC3 audio support" & vbCrLf
                End If
                Select Case AVSAudioInput
                Case "AVISource"
                    AudioStr = AudioStr & "Audio = AVISource(""" & .InputAudioPathName(AudioNum) & """)" & vbCrLf
                Case "DirectShowSource"
                    AudioStr = AudioStr & "Audio = DirectShowSource(""" & .InputAudioPathName(AudioNum) & """, video=false)" & vbCrLf
                Case "FFMpegSource"
                    AudioStr = AudioStr & "Audio = FFAudioSource(""" & .InputAudioPathName(AudioNum) & """, track=" & TrackNum & ")" & vbCrLf
                Case "FFAudioSource (forced)"
                    Randomize
                    AudioStr = AudioStr & "Audio = FFAudioSource(""" & .InputAudioPathName(AudioNum) & """, track=" & TrackNum & ", cachefile=""" & Preferences.TempPath & "\A2DFFA_"" + String(Rand()) + "".cache""" & ")" & vbCrLf
                Case Else
                    AudioStr = AudioStr & "Audio = A2DAudioSource(""" & .InputAudioPathName(AudioNum) & """, CacheFolder=""" & Preferences.TempPath & """)" & vbCrLf
                End Select
            Else
                AudioStr = AudioStr & "Audio = Import(""" & .InputAudioPathName(AudioNum) & """)" & vbCrLf
            End If
        End If
                
        ' Write Video and Audio Sources
        If Preferences.AVSAudioInput = "FFMpegSource" Or Preferences.AVSVideoInput = "FFMpegSource" Then
            Create_AVSScript = Create_AVSScript & AudioStr & VideoStr & vbCrLf
        Else
            Create_AVSScript = Create_AVSScript & VideoStr & AudioStr & vbCrLf
        End If
        
        'VIDEO EDIT SECTION
        
        ' Color Space conversion
        Create_AVSScript = Create_AVSScript & "Video = Video.ConvertToYV12"
        If InputInterlaced = True Then
            Create_AVSScript = Create_AVSScript & "(interlaced=true)" & vbCrLf
        Else
            Create_AVSScript = Create_AVSScript & "()" & vbCrLf   'http://forum.doom9.org/showthread.php?p=1588645#post1588645
        End If
        If .AVSColorMatrix = True Then
            If InputInterlaced = True Then
                Create_AVSScript = Create_AVSScript & "Video = Video.AssumeFrameBased().ColorMatrix(source=" & .AVS.ColorMatrixSource & ",dest=" & .AVS.ColorMatrixDest & ",interlaced=true)" & vbCrLf
            Else
                Create_AVSScript = Create_AVSScript & "Video = Video.ColorMatrix(source=" & .AVS.ColorMatrixSource & ",dest=" & .AVS.ColorMatrixDest & ")" & vbCrLf
            End If
        End If
        
        ' Add DeBlock
        If .AVSDeBlock = True Then
            Str = ""
            dY = 0
            dX = 0
            With .InputVideoInfo
                If modNum.IsMultipleN(Int(Val(.Height)), 8) = False Then
                    dY = (Val(.Height) / 8 - Int(Val(.Height) / 8)) * 4
                    'Create_AVSScript = Create_AVSScript & "Video = Video.AddBorders(0," & dY & ",0," & dY & ")" & vbCrLf
                End If
                If modNum.IsMultipleN(Int(Val(.Width)), 8) = False Then
                    dX = (Val(.Width) / 8 - Int(Val(.Width) / 8)) * 4
                    'Create_AVSScript = Create_AVSScript & "Video = Video.AddBorders(" & dX & ",0," & dX & ",0)" & vbCrLf
                End If
                If dX <> 0 Or dY <> 0 Then
                    Str = "AddBorders(" & dX & "," & dY & "," & dX & "," & dY & ")."
                End If
            End With
            If InputInterlaced <> 0 Then
                Str = Str & "DeBlock_Int"
            Else
                Str = Str & "Deblock"
            End If
            Str = Str & "(" & .AVS.DeBlock & ")"
            If dX <> 0 Or dY <> 0 Then
                Str = Str & ".Crop(" & dX & "," & dY & "," & -dX & "," & -dY & ")"
            End If
            Create_AVSScript = Create_AVSScript & "Video = Video." & Str & vbCrLf
        End If
        
        ' Add deinterlace
        If .AVSDeInterlace = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video.LeakKernelDeint(" & IIf(.VideoFieldOrder = "BFF", "0", "1") & "," & .AVS.DeIntThreshold & ",false,false)" & vbCrLf
        End If
        
        ' Adjust Video Aspect -> generate ResizeStr temp script
        ResizeStr = ""
        X = Val(.InputVideoInfo.Width)
        If X = 0 Then X = DVD_H
        Y = Round(X / InputDAR * IIf(.AVSVerticalResize = False, 100, Val(.AVS.VResize)) / 100, 0)
        If .AVSCrop = True Then
            ResizeStr = "Video = Video.Crop(" & .AVS.CropSx & "," & .AVS.CropUp & "," & -.AVS.CropDx & "," & -.AVS.CropDo & ")" & vbCrLf
            X = X - .AVS.CropSx - .AVS.CropDx
            Y = Y - .AVS.CropUp - .AVS.CropDo
        End If
        Select Case X / IIf(.VideoAspectRatio = Standard, 4 / 3, 16 / 9)
        Case Is > Y
            ' Add top and bottom letterboxes
            dY = Int((X / IIf(.VideoAspectRatio = Standard, 4 / 3, 16 / 9) - Y) / 2 * Val(.InputVideoInfo.PAR))
            If dY > 1 Then
                ResizeStr = ResizeStr & "Video = Video.AddBorders(0," & dY + CInt(IsOdd(dY)) & ",0," & dY - CInt(IsOdd(dY)) & ")" & vbCrLf
            End If
            ResizeStr = ResizeStr & "Video = Video." & ResizeFilter & "(" & DVD_H & "," & DVD_V & IIf(ResizeFilter = "BicubicResize", ",0,0.5", "") & ")"
        Case Is < Y
            'If .VideoAspectRatio = Widescreen Then
            If Preferences.AVSAdjustFrame = CropTopBottom Then
                ' Crop top and bottom letterboxes
                dY = Int((Y - X / 16 * 9) / 2)
                dY = IIf(IsEven((dY)) = True, dY, dY - 1)
                ResizeStr = ResizeStr & "Video = Video." & ResizeFilter & "(" & DVD_H & "," & DVD_V & IIf(ResizeFilter = "BicubicResize", ",0,0.5", "") & ",0," & dY & ",0,-" & dY & ")"
            Else
                ' Add left and right letterboxes
                If .InputVideoInfo.DVDCompliant = Not_Video_Compl Then
                    dX = Int((Y * IIf(.VideoAspectRatio = Standard, 4 / 3, 16 / 9) - X) / 2)
                    If dX > 1 Then
                        ResizeStr = ResizeStr & "Video = Video.AddBorders(" & dX + CInt(IsOdd(dX)) & ",0," & dX - CInt(IsOdd(dX)) & ",0)" & vbCrLf
                    End If
                End If
                ResizeStr = ResizeStr & "Video = Video." & ResizeFilter & "(" & DVD_H & "," & DVD_V & IIf(ResizeFilter = "BicubicResize", ",0,0.5", "") & ")"
            End If
        Case Else
            If X <> DVD_H Or Val(.InputVideoInfo.Height) <> DVD_V Then
                ResizeStr = ResizeStr & "Video = Video." & ResizeFilter & "(" & DVD_H & "," & DVD_V & IIf(ResizeFilter = "BicubicResize", ",0,0.5", "") & ")"
            End If
        End Select
        If dX = 0 And dY = 0 And DVD_V = Val(.InputVideoInfo.Height) And DVD_H = Val(.InputVideoInfo.Width) And .AVSCrop = False Then
            ResizeStr = ""
        End If
        
        ' Bob interlaced clip before resizing
        If .AVSDeInterlace = False Then
            If ResizeStr <> "" And InputInterlaced = True Then
                Create_AVSScript = Create_AVSScript & "Video = Video.LeakKernelBob(" & IIf(.VideoFieldOrder = "BFF", "0", "1") & ",7,false,false)" & vbCrLf
            End If
        End If
        
        ' Denoise
        If .AVSHQDN3D = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video.hqdn3d(" & .AVS.HQDN3DStrength & ")" & vbCrLf
        End If
        
        ' Resize
        If ResizeStr <> "" Then
            Create_AVSScript = Create_AVSScript & ResizeStr & vbCrLf
        End If
        
        ' Sharpen
        If .AVSSharpen = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video.Sharpen(" & Replace(.AVS.Sharpness, ",", ".") & ",MMX=false)" & vbCrLf
        End If
        If .AVSMSharpen = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video.MSharpen(" & .AVS.MSharpenThreshold & "," & .AVS.MSharpenStrength & ")" & vbCrLf
        End If
        
        ' Weave after video resizing
        If InStr(1, Create_AVSScript, "LeakKernelBob(") <> 0 Then
            Create_AVSScript = Create_AVSScript & "Video = Video.SeparateFields().SelectEvery(4," & IIf(.VideoFieldOrder = "BFF", "0,3", "1,2") & ").Weave()" & vbCrLf
        End If
    
        ' Adjust FPS
        If FPS < 2 / 3 * InputFPS Then
            Str = "Video = Video.ChangeFPS(""" & VideoStandardStr & """)" & vbCrLf
        Else
            Str = "Video = Video.ConvertFPS(""" & VideoStandardStr & """)" & vbCrLf
        End If
        If .InputVideoInfo.FrameRateMode = "CFR" Then
            Select Case Round(InputFPS - FPS, 3)
            Case Is = 0
                Str = ""
            Case Is > 0
                If InputFPS = 2 * FPS Then
                    Str = "Video = Video.SelectEven()" & vbCrLf
                Else
                    Str = "Video = Video.ChangeFPS(""" & VideoStandardStr & """)" & vbCrLf
                End If
            Case Is < 0
                Select Case InputFPS
                Case 23.976, 24, 25, 29.97, 30, 50, 59.94, 60    'only these fps are mpeg2 compliant
                    Str = "#Using DGPulldown/HCenc to upsize FPS" & vbCrLf
                End Select
                If Preferences.PALSpeedUp = True And FPS = 25 And InputFPS = 23.976 Then
                    Str = "#Using AssumeFPS() to upsize FPS" & vbCrLf    'see below
                End If
            End Select
        End If
        Create_AVSScript = Create_AVSScript & Str
        
        ' Tweak Colors
        If .AVSTweak = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video.Tweak(" & Replace(.AVS.Hue, ",", ".") & "," & _
                                                                           Replace(.AVS.Saturation, ",", ".") & "," & _
                                                                           Replace(.AVS.Brightness, ",", ".") & "," & _
                                                                           Replace(.AVS.Contrast, ",", ".") & ")" & vbCrLf
        End If
        
        ' Insert subs
        If .InputSubsPathName(1) <> "" And .SubsHardEncode(1) = True Then
            Create_AVSScript = Create_AVSScript & "Video = Video"
            Select Case UCase(modFS.ExtOf(.InputSubsPathName(1)))
            Case "SRT", "SSA", "ASS"
                Create_AVSScript = Create_AVSScript & ".TextSub("
            Case "IDX"
                Create_AVSScript = Create_AVSScript & ".VobSub("
            End Select
            Create_AVSScript = Create_AVSScript & """" & .InputSubsPathName(1) & """)" & vbCrLf
        End If

        Create_AVSScript = Create_AVSScript & vbCrLf


        'AUDIO EDIT SECTION
        
        If .InputAudioPathName(AudioNum) <> "" And .AudioIsToBeKept(AudioNum) = False Then
            ' Adjust Audio settings
            With .InputAudioInfo(AudioNum)
                ' Handle MP3 VBR
                If Left(.Compression, 2) & Right(.Compression, 1) = "MP3" And .BitRateMode = "VBR" Then
                    Create_AVSScript = Create_AVSScript & "Audio = Audio.EnsureVBRMP3Sync()" & vbCrLf
                End If
                ' Adjust SampleRate
                Flag = False
                If Val(.SampleRate) <> 48000 Then Flag = True
            End With
            If .InputVideoPathName = .InputAudioPathName(AudioNum) And .InputVideoInfo.Compression = "DV" Then Flag = True
            If Flag = True Then
                Create_AVSScript = Create_AVSScript & "Audio = Audio.SSRC(48000)" & vbCrLf
            End If
            ' Downmix if necessary
            Select Case .AudioChannelsMode(AudioNum)
            Case "1.0"
                Create_AVSScript = Create_AVSScript & "Audio = Audio.ConvertToMono" & vbCrLf
            Case "2.0"
                If Val(.InputAudioInfo(AudioNum).Channels) = 6 And Preferences.AVSAudioInput = "FFMpegSource" Then
                    'Create_AVSScript = Create_AVSScript & _
                        "flr = GetChannel(Audio, 1, 2)" & vbCrLf & _
                        "fcc = GetChannel(Audio, 3, 3)" & vbCrLf & _
                        "lrc = MixAudio(flr, fcc, 0.3694, 0.2612)" & vbCrLf & _
                        "blr = GetChannel(Audio, 5, 6)" & vbCrLf & _
                        "Audio = MixAudio(lrc, blr, 1.0, 0.3694)" & vbCrLf
                    Create_AVSScript = Create_AVSScript & _
                        "Audio = Audio.Dmix6Stereo()" & vbCrLf
                End If
            End Select
            ' Normalize
            If .AudioNormalize(AudioNum) = True Then
                Str = Replace(Preferences.AudioNormalizeVal, ",", ".")
                If Val(Str) = 1 Then Str = ""
                Create_AVSScript = Create_AVSScript & "Audio = Audio.Normalize(" & Str & ")" & vbCrLf
            End If
            ' Auto Delay
            Str = .InputAudioInfo(AudioNum).Delay
            If .AudioDelayAuto(AudioNum) = True And Val(Str) <> 0 Then
                Create_AVSScript = Create_AVSScript & "Audio = Audio.DelayAudio(" & Replace(Round(Val(Str) / 1000, 3), ",", ".") & ")" & vbCrLf
            End If
            ' Apply AviSynth filters from AVS variables (audio)
            If .AVSDelayAudio = True Then
                Create_AVSScript = Create_AVSScript & "Audio = Audio.DelayAudio(" & Replace(.AVS.AudioDelay, ",", ".") & ")" & vbCrLf
            End If
            If .AVSAmplifyAudio = True Then
                Create_AVSScript = Create_AVSScript & "Audio = Audio.AmplifyDB(" & Replace(.AVS.AudioAmplify, ",", ".") & ")" & vbCrLf
            End If
            If InStr(1, Create_AVSScript, "Audio = Audio.") <> 0 Then
                Create_AVSScript = Create_AVSScript & vbCrLf
            End If
        End If
        
        
        ' FINAL SECTION
        
        ' Mux Video and Audio or Return Video Only + PAL SpeedUp (if any)
        If InStr(1, Create_AVSScript, "Video =") <> 0 And InStr(1, Create_AVSScript, "Audio =") <> 0 Then
            Create_AVSScript = Create_AVSScript & "AudioDub(Video, Audio)" & vbCrLf
            If Preferences.PALSpeedUp = True And InStr(1, Create_AVSScript, "#Using AssumeFPS() to upsize FPS") <> 0 Then
                Create_AVSScript = modStr.RemoveStr(Create_AVSScript, "Audio = Audio.SSRC(48000)" & vbCrLf & vbCrLf)
                Create_AVSScript = modStr.RemoveStr(Create_AVSScript, "Audio = Audio.SSRC(48000)" & vbCrLf)
                Create_AVSScript = Create_AVSScript & "AssumeFPS(""pal_film"", sync_audio=true)"
                Select Case .InputAudioInfo(AudioNum).SampleRate
                Case "48000"
                    Create_AVSScript = Create_AVSScript & ".SSRC(48000)" & vbCrLf
                Case Else  'SSRC does not accept all input samplerates
                    Create_AVSScript = Create_AVSScript & ".ResampleAudio(48000)" & vbCrLf
                End Select
            End If
        Else
            If Preferences.PALSpeedUp = True And InStr(1, Create_AVSScript, "#Using AssumeFPS() to upsize FPS") <> 0 Then
                Create_AVSScript = Create_AVSScript & "Video = Video.AssumeFPS(""pal_film"")" & vbCrLf
            End If
            Create_AVSScript = Create_AVSScript & "Return Video" & vbCrLf
        End If
        Create_AVSScript = Create_AVSScript & vbCrLf
        
        ' Apply AviSynth filters from AVS variables (general)
        Str = ""
        If .AVSTrim = True Then
            Str = "Trim(" & .AVS.FrameStart & "," & .AVS.FrameEnd & ")"
            If InStr(1, Create_AVSScript, "Return Video" & vbCrLf) <> 0 Then
                Create_AVSScript = Replace(Create_AVSScript, "Return Video" & vbCrLf, "Return Video." & Str & vbCrLf)
            Else
                Create_AVSScript = Create_AVSScript & Str & vbCrLf
            End If
        End If
        If .AVSFadeIO = True Then
            Str = "FadeIO(" & .AVS.FadeIOFrames & ")"
            If InStr(1, Create_AVSScript, "Return Video" & vbCrLf) <> 0 Then
                Create_AVSScript = Replace(Create_AVSScript, "Return Video" & vbCrLf, "Return Video." & Str & vbCrLf)
            Else
                Create_AVSScript = Create_AVSScript & Str & vbCrLf
            End If
        End If
        
    End With

    ' Append on top the Plugin calls
    If InStr(1, Create_AVSScript, "A2DVideoSource(") <> 0 Or InStr(1, Create_AVSScript, "A2DAudioSource(") <> 0 Then
        sPlugin = sPlugin & "Import(""" & App.Path & "\Lib\A2DSource.avsi""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "FFVideoSource(") <> 0 Or InStr(1, Create_AVSScript, "FFAudioSource(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\Lib\ffms2.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "MPEG2Source(") <> 0 Or InStr(1, Create_AVSScript, "Deblock(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\DGIndex\DGDecode.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "DeBlock_Int(") <> 0 Then
        sPlugin = sPlugin & "Import(""" & App.Path & "\Lib\DeBlock_Int.avsi""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "TextSub(") <> 0 Or InStr(1, Create_AVSScript, "VobSub(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\VobSub\VSFilter.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "LeakKernelBob(") <> 0 Or InStr(1, Create_AVSScript, "LeakKernelDeint(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\Lib\LeakKernelDeint.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "ColorMatrix(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\Lib\ColorMatrix.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "hqdn3d(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\Lib\hqdn3d.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "MSharpen(") <> 0 Then
        sPlugin = sPlugin & "LoadPlugin(""" & App.Path & "\Lib\MSharpen.dll""" & ")" & vbCrLf
    End If
    If InStr(1, Create_AVSScript, "Dmix6Stereo(") <> 0 Then
        sPlugin = sPlugin & "Import(""" & App.Path & "\Lib\Downmix.avsi""" & ")" & vbCrLf
    End If
    If sPlugin <> "" Then Create_AVSScript = sPlugin & vbCrLf & Create_AVSScript
    
    ' Remove junk stuff
    Create_AVSScript = Replace(Create_AVSScript, "Video = Video." & ResizeFilter & "(720,576,0,0,0,-0)", "Video = Video." & ResizeFilter & "(720,576)")
    Create_AVSScript = modStr.RemoveStr(Create_AVSScript, "Video = Video.AddBorders(0,0,0,0)" & vbCrLf)
    'If InStr(1, Create_AVSScript, "Video.LeakKernel") = 0 Then
    '    Create_AVSScript = modStr.RemoveStr(Create_AVSScript, "LoadPlugin(""" & App.Path & "\Lib\LeakKernelDeint.dll""" & ")" & vbCrLf)
    'End If
    Create_AVSScript = modStr.RemoveCharBegEnd(Create_AVSScript, Chr(10), Chr(13))
    
    Exit Function

Err_Handling:
Select Case Err.Number
Case 999
    Resume
Case Else
    Create_AVSScript = ""
End Select

End Function
